package zero1hd.polyjet.audio;

import java.io.IOException;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;

import org.jaudiotagger.audio.exceptions.InvalidAudioFrameException;
import org.jaudiotagger.audio.exceptions.ReadOnlyFileException;
import org.jaudiotagger.audio.mp3.MP3File;
import org.jaudiotagger.tag.TagException;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.files.FileHandle;

public class Mp3AudioData implements AudioData {
	private int readWindowSize = 1024;
	
	private AudioInputStream audStream;
	private AudioInputStream faudStream;
	
	private AudioFormat audioFormat;
	private Music playbackMusic;
	private int readIndex;
	
	private int sampleCount;
	public Mp3AudioData(FileHandle audioFile) {
		try {
			sampleCount = (int) new MP3File(audioFile.file()).getMP3AudioHeader().getNumberOfFrames();
		} catch (IOException | TagException | ReadOnlyFileException | InvalidAudioFrameException e1) {
			e1.printStackTrace();
		}
		
		reset();
		try {
			audStream = AudioSystem.getAudioInputStream(audioFile.file());
			AudioFormat bFormat = audStream.getFormat();
			audioFormat = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, bFormat.getSampleRate(), 16, bFormat.getChannels(), bFormat.getChannels()*2, bFormat.getSampleRate(), false);
			faudStream = AudioSystem.getAudioInputStream(audioFormat, audStream);
		} catch (UnsupportedAudioFileException | IOException e) {
			e.printStackTrace();
		}
		
		
		playbackMusic = Gdx.audio.newMusic(audioFile);
	}
	
	
	@Override
	public void readIndexUpdate() {
		readIndex = (int) (playbackMusic.getPosition() * audioFormat.getSampleRate() / readWindowSize);
	}

	@Override
	public int getReadIndex() {
		return readIndex;
	}

	@Override
	public void reset() {
		if (playbackMusic != null) {
			playbackMusic.stop();
			playbackMusic.dispose();
			playbackMusic = null;
		}

		if (audStream != null) {
			try {
				audStream.close();
				faudStream.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Override
	public int getReadWindowSize() {
		return readWindowSize;
	}

	@Override
	public Music getPlaybackMusic() {
		return playbackMusic;
	}

	@Override
	public int readSamples(float[] samples) {
		int samplesRead = 0;
		int sampleAverage = 0;
		for (int currentSample = 0; currentSample < samples.length; currentSample++) {
			for (int channel = 0; channel < audioFormat.getChannels(); channel++) {
				try {
					sampleAverage += audStream.read();
					if (sampleAverage == -1) {
						break;
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			sampleAverage /= audioFormat.getChannels()*Short.MAX_VALUE+1;
			samples[currentSample] = sampleAverage;
			samplesRead++;
			if (sampleAverage == -1) {
				break;
			}
		}
		return samplesRead;
	}

	@Override
	public AudioFormat getFormat() {
		return audioFormat;
	}


	@Override
	public int getSampleCount() {
		return sampleCount;
	}

}
